@@ -32,47 +32,49 @@ from reliability.Distributions import Weibull_Distribution, Lognormal_Distributi
 from reliability.Nonparametric import KaplanMeier, NelsonAalen
 
 np.seterr('ignore')
+dec = 3  # number of decimals to use when rounding fitted parameters in labels
 
+from reliability.Utils import axes_transforms, round_to_decimals
 
-# Custom scale functions
-def __weibull_forward(F):
-    return np.log(-np.log(1 - F))
-
-
-def __weibull_inverse(R):
-    return 1 - np.exp(-np.exp(R))
-
-
-def __expon_forward(F):
-    return ss.expon.ppf(F)
-
-
-def __expon_inverse(R):
-    return ss.expon.cdf(R)
-
-
-def __normal_forward(F):
-    return ss.norm.ppf(F)
-
-
-def __normal_inverse(R):
-    return ss.norm.cdf(R)
-
-
-def __gamma_forward(F, beta):
-    return ss.gamma.ppf(F, a=beta)
-
-
-def __gamma_inverse(R, beta):
-    return ss.gamma.cdf(R, a=beta)
-
-
-def __beta_forward(F, alpha, beta):
-    return ss.beta.ppf(F, a=alpha, b=beta)
-
-
-def __beta_inverse(R, alpha, beta):
-    return ss.beta.cdf(R, a=alpha, b=beta)
+# # Custom scale functions
+# def __weibull_forward(F):
+#     return np.log(-np.log(1 - F))
+#
+#
+# def __weibull_inverse(R):
+#     return 1 - np.exp(-np.exp(R))
+#
+#
+# def __expon_forward(F):
+#     return ss.expon.ppf(F)
+#
+#
+# def __expon_inverse(R):
+#     return ss.expon.cdf(R)
+#
+#
+# def __normal_forward(F):
+#     return ss.norm.ppf(F)
+#
+#
+# def __normal_inverse(R):
+#     return ss.norm.cdf(R)
+#
+#
+# def __gamma_forward(F, beta):
+#     return ss.gamma.ppf(F, a=beta)
+#
+#
+# def __gamma_inverse(R, beta):
+#     return ss.gamma.cdf(R, a=beta)
+#
+#
+# def __beta_forward(F, alpha, beta):
+#     return ss.beta.ppf(F, a=alpha, b=beta)
+#
+#
+# def __beta_inverse(R, alpha, beta):
+#     return ss.beta.cdf(R, a=alpha, b=beta)
 
 
 def plotting_positions(failures=None, right_censored=None, h1=None, h2=None):
@@ -148,7 +150,7 @@ def plotting_positions(failures=None, right_censored=None, h1=None, h2=None):
     return x, y
 
 
-def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
+def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, CI=0.95, CI_type='time', show_fitted_distribution=True, **kwargs):
     '''
     Weibull probability plot
 
@@ -160,6 +162,8 @@ def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False
     show_fitted_distribution - True/False. If true, the fitted distribution will be plotted on the probability plot. Defaults to True
     h1 and h2 are the heuristic constants for plotting positions of the form (k-h1)/(n+h2). Default is h1=0.3,h2=0.4 which is the median rank method (same as in Minitab).
         For more heuristics, see: https://en.wikipedia.org/wiki/Q%E2%80%93Q_plot#Heuristics
+    CI - the confidence interval for the bounds. Default is 0.95 for 95% CI.
+    CI_type - 'time' or 'reliability'. Default is 'time'. This is the type of CI bounds. i.e. bounds on time or bounds on reliability.
     kwargs are accepted for the fitted line (eg. linestyle, label, color)
 
     Outputs:
@@ -183,9 +187,9 @@ def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False
             raise ValueError('right_censored must be a list or an array')
     # generate the figure and fit the distribution
     if max(failures) < 1:
-        xvals = np.linspace(10 ** -5, 2, 1000)
+        xvals = np.linspace(10 ** -10, 3, 1000)
     else:
-        xvals = np.logspace(-4, np.ceil(np.log10(max(failures))) + 1, 1000)
+        xvals = np.logspace(-25, np.ceil(np.log10(max(failures))) + 6, 1000)
 
     if __fitted_dist_params is not None:
         if __fitted_dist_params.gamma > 0:
@@ -195,53 +199,72 @@ def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False
         if __fitted_dist_params is not None:
             alpha = __fitted_dist_params.alpha
             beta = __fitted_dist_params.beta
+            gamma = 0
+            alpha_SE = __fitted_dist_params.alpha_SE
+            beta_SE = __fitted_dist_params.beta_SE
+            gamma_SE = 0
+            Cov_alpha_beta = __fitted_dist_params.Cov_alpha_beta
+            Cov_alpha_gamma = 0
+            Cov_beta_gamma = 0
         else:
             from reliability.Fitters import Fit_Weibull_2P
-            fit = Fit_Weibull_2P(failures=failures, right_censored=right_censored, show_probability_plot=False, print_results=False)
+            fit = Fit_Weibull_2P(failures=failures, right_censored=right_censored, CI=CI, show_probability_plot=False, print_results=False)
             alpha = fit.alpha
             beta = fit.beta
-        wbf = Weibull_Distribution(alpha=alpha, beta=beta).CDF(show_plot=False, xvals=xvals)
+            gamma = 0
+            alpha_SE = fit.alpha_SE
+            beta_SE = fit.beta_SE
+            gamma_SE = 0
+            Cov_alpha_beta = fit.Cov_alpha_beta
+            Cov_alpha_gamma = 0
+            Cov_beta_gamma = 0
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Weibull_2P (α=' + str(round(alpha, 2)) + ', β=' + str(round(beta, 2)) + ')')
+            label = str('Fitted Weibull_2P (α=' + str(round_to_decimals(alpha, dec)) + ', β=' + str(round_to_decimals(beta, dec)) + ')')
         if 'color' in kwargs:
-            color = kwargs.pop('color')
-            data_color = color
+            data_color = kwargs.get('color')
         else:
-            color = 'red'
             data_color = 'k'
-        plt.xlabel('Time')
+        xlabel = 'Time'
     elif fit_gamma is True:
         if __fitted_dist_params is not None:
             alpha = __fitted_dist_params.alpha
             beta = __fitted_dist_params.beta
             gamma = __fitted_dist_params.gamma
+            alpha_SE = __fitted_dist_params.alpha_SE
+            beta_SE = __fitted_dist_params.beta_SE
+            gamma_SE = __fitted_dist_params.gamma_SE
+            Cov_alpha_beta = __fitted_dist_params.Cov_alpha_beta
         else:
             from reliability.Fitters import Fit_Weibull_3P
-            fit = Fit_Weibull_3P(failures=failures, right_censored=right_censored, show_probability_plot=False, print_results=False)
+            fit = Fit_Weibull_3P(failures=failures, right_censored=right_censored, CI=CI, show_probability_plot=False, print_results=False)
             alpha = fit.alpha
             beta = fit.beta
             gamma = fit.gamma
-        wbf = Weibull_Distribution(alpha=alpha, beta=beta).CDF(show_plot=False, xvals=xvals)
+            alpha_SE = fit.alpha_SE
+            beta_SE = fit.beta_SE
+            gamma_SE = fit.gamma_SE
+            Cov_alpha_beta = fit.Cov_alpha_beta
+
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Weibull_3P\n(α=' + str(round(alpha, 2)) + ', β=' + str(round(beta, 2)) + ', γ=' + str(round(gamma, 2)) + ')')
+            label = str('Fitted Weibull_3P\n(α=' + str(round_to_decimals(alpha, dec)) + ', β=' + str(round_to_decimals(beta, dec)) + ', γ=' + str(round_to_decimals(gamma, dec)) + ')')
         if 'color' in kwargs:
-            color = kwargs.pop('color')
-            data_color = color
+            data_color = kwargs.get('color')
         else:
-            color = 'red'
             data_color = 'k'
-        plt.xlabel('Time - gamma')
+        xlabel = 'Time - gamma'
         failures = failures - gamma
         if right_censored is not None:
             right_censored = right_censored - gamma
+    wbf = Weibull_Distribution(alpha=alpha, beta=beta, alpha_SE=alpha_SE, beta_SE=beta_SE, Cov_alpha_beta=Cov_alpha_beta, CI=CI, CI_type=CI_type)
+
     # plot the failure points and format the scale and axes
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)
     plt.scatter(x, y, marker='.', linewidth=2, c=data_color)
-    plt.gca().set_yscale('function', functions=(__weibull_forward, __weibull_inverse))
+    plt.gca().set_yscale('function', functions=(axes_transforms.weibull_forward, axes_transforms.weibull_inverse))
     plt.xscale('log')
     plt.grid(b=True, which='major', color='k', alpha=0.3, linestyle='-')
     plt.grid(b=True, which='minor', color='k', alpha=0.08, linestyle='-')
@@ -249,6 +272,8 @@ def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False
     xrange = plt.gca().get_xlim()  # this ensures the previously plotted objects are considered when setting the range
     xrange_min = min(min(x), xrange[0])
     xrange_max = max(max(x), xrange[1])
+    if xrange_min <=0:
+        xrange_min=1e-2
     pts_min_log = 10 ** (int(np.floor(np.log10(xrange_min))))  # second smallest point is rounded down to nearest power of 10
     pts_max_log = 10 ** (int(np.ceil(np.log10(xrange_max))))  # largest point is rounded up to nearest power of 10
     plt.xlim([pts_min_log, pts_max_log])
@@ -256,15 +281,17 @@ def Weibull_probability_plot(failures=None, right_censored=None, fit_gamma=False
     ytickvals = [0.0001, 0.0003, 0.001, 0.002, 0.003, 0.005, 0.01, 0.02, 0.03, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.95, 0.99, 0.999, 0.9999]
     plt.yticks(ytickvals)
     plt.gca().set_yticklabels(['{:,.2%}'.format(x) for x in ytickvals])  # formats y ticks as percentage
-    plt.title('Probability plot\nWeibull CDF')
-    plt.ylabel('Fraction failing')
+
     plt.gcf().set_size_inches(9, 7)  # adjust the figsize. This is done post figure creation so that layering is easier
     if show_fitted_distribution is True:
-        plt.plot(xvals, wbf, color=color, label=label, **kwargs)
+        wbf.CDF(xvals=xvals,label=label, **kwargs)
         plt.legend(loc='upper left')
+    plt.title('Probability plot\nWeibull CDF')
+    plt.ylabel('Fraction failing')
+    plt.xlabel(xlabel) #### needs to be set after plotting the CDF to override the default 'xvals'
+    return plt.gcf()
 
-
-def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
+def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, CI=0.95, show_fitted_distribution=True, **kwargs):
     '''
     Exponential probability plot Weibull Scale
 
@@ -279,6 +306,7 @@ def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=Non
     show_fitted_distribution - True/False. If true, the fitted distribution will be plotted on the probability plot. Defaults to True
     h1 and h2 are the heuristic constants for plotting positions of the form (k-h1)/(n+h2). Default is h1=0.3,h2=0.4 which is the median rank method (same as in Minitab).
         For more heuristics, see: https://en.wikipedia.org/wiki/Q%E2%80%93Q_plot#Heuristics
+    CI - the confidence interval for the bounds. Default is 0.95 for 95% CI.
     kwargs are accepted for the fitted line (eg. linestyle, label, color)
 
     Outputs:
@@ -302,9 +330,9 @@ def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=Non
             raise ValueError('right_censored must be a list or an array')
     # generate the figure and fit the distribution
     if max(failures) < 1:
-        xvals = np.linspace(10 ** -5, 2, 1000)
+        xvals = np.logspace(-5, 1, 1000)
     else:
-        xvals = np.logspace(-4, np.ceil(np.log10(max(failures))) + 1, 1000)
+        xvals = np.logspace(np.floor(np.log10(min(failures))) - 3, np.ceil(np.log10(max(failures))) + 1, 1000)
 
     if __fitted_dist_params is not None:
         if __fitted_dist_params.gamma > 0:
@@ -313,50 +341,57 @@ def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=Non
     if fit_gamma is False:
         if __fitted_dist_params is not None:
             Lambda = __fitted_dist_params.Lambda
+            Lambda_SE = __fitted_dist_params.Lambda_SE ####
         else:
             from reliability.Fitters import Fit_Expon_1P
-            fit = Fit_Expon_1P(failures=failures, right_censored=right_censored, show_probability_plot=False, print_results=False)
+            fit = Fit_Expon_1P(failures=failures, right_censored=right_censored, CI=CI, show_probability_plot=False, print_results=False)
             Lambda = fit.Lambda
-        ef = Exponential_Distribution(Lambda=Lambda).CDF(show_plot=False, xvals=xvals)
+            Lambda_SE = fit.Lambda_SE ####
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Exponential_1P (λ=' + str(round(Lambda, 8)) + ')')
-        if 'color' in kwargs:
-            color = kwargs.pop('color')
-            data_color = color
-        else:
-            color = 'red'
-            data_color = 'k'
-        plt.xlabel('Time')
+            label = str('Fitted Exponential_1P (λ=' + str(round_to_decimals(Lambda, dec)) + ')')
+        if 'color' in kwargs: ####
+            data_color = kwargs.get('color') ####
+        else: ####
+            data_color = 'k' ####
+        xlabel = 'Time' ####
     elif fit_gamma is True:
         if __fitted_dist_params is not None:
             Lambda = __fitted_dist_params.Lambda
-            gamma = __fitted_dist_params.gamma
+            Lambda_SE = __fitted_dist_params.Lambda_SE ####
+            gamma = __fitted_dist_params.gamma ####
         else:
             from reliability.Fitters import Fit_Expon_2P
-            fit = Fit_Expon_2P(failures=failures, right_censored=right_censored, show_probability_plot=False, print_results=False)
+            fit = Fit_Expon_2P(failures=failures, right_censored=right_censored, CI=CI, show_probability_plot=False, print_results=False)
             Lambda = fit.Lambda
-            gamma = fit.gamma
-        ef = Exponential_Distribution(Lambda=Lambda).CDF(show_plot=False, xvals=xvals)
+            Lambda_SE = fit.Lambda_SE ####
+            gamma = fit.gamma ####
+
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Exponential_2P\n(λ=' + str(round(Lambda, 8)) + ', γ=' + str(round(gamma, 2)) + ')')
-        if 'color' in kwargs:
-            color = kwargs.pop('color')
-            data_color = color
-        else:
-            color = 'red'
-            data_color = 'k'
-        plt.xlabel('Time - gamma')
+            label = str('Fitted Exponential_2P\n(λ=' + str(round_to_decimals(Lambda, dec)) + ', γ=' + str(round_to_decimals(gamma, dec)) + ')')
+        if 'color' in kwargs: ####
+            data_color = kwargs.get('color') ####
+        else: ####
+            data_color = 'k' ####
+        xlabel = 'Time - gamma' ####
         failures = failures - gamma + 0.009  # this 0.009 adjustment is to avoid taking the log of 0. It causes negligible difference to the fit and plot. 0.009 is chosen to be the same as Weibull_Fit_3P adjustment.
         if right_censored is not None:
             right_censored = right_censored - gamma + 0.009  # this 0.009 adjustment is to avoid taking the log of 0. It causes negligible difference to the fit and plot. 0.009 is chosen to be the same as Weibull_Fit_3P adjustment.
+
+        #### recalculate the xvals for the plotting range when gamma>0
+        if max(failures)-gamma < 1:
+            xvals = np.logspace(-5, 1, 1000)
+        else:
+            xvals = np.logspace(-4, np.ceil(np.log10(max(failures)-gamma)) + 1, 1000) ####needed to adjust the lower lim here so it is > 0
+    ef = Exponential_Distribution(Lambda=Lambda, Lambda_SE=Lambda_SE, CI=CI) ####added extra params and removed .CDF
+
     # plot the failure points and format the scale and axes
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)
     plt.scatter(x, y, marker='.', linewidth=2, c=data_color)
-    plt.gca().set_yscale('function', functions=(__weibull_forward, __weibull_inverse))
+    plt.gca().set_yscale('function', functions=(axes_transforms.weibull_forward, axes_transforms.weibull_inverse))
     plt.xscale('log')
     plt.grid(b=True, which='major', color='k', alpha=0.3, linestyle='-')
     plt.grid(b=True, which='minor', color='k', alpha=0.08, linestyle='-')
@@ -364,6 +399,8 @@ def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=Non
     xrange = plt.gca().get_xlim()  # this ensures the previously plotted objects are considered when setting the range
     xrange_min = min(min(x), xrange[0])
     xrange_max = max(max(x), xrange[1])
+    if xrange_min <=0:
+        xrange_min=1e-2
     pts_min_log = 10 ** (int(np.floor(np.log10(xrange_min))))  # second smallest point is rounded down to nearest power of 10
     pts_max_log = 10 ** (int(np.ceil(np.log10(xrange_max))))  # largest point is rounded up to nearest power of 10
     plt.xlim([pts_min_log, pts_max_log])
@@ -371,13 +408,14 @@ def Exponential_probability_plot_Weibull_Scale(failures=None, right_censored=Non
     ytickvals = [0.0001, 0.0003, 0.001, 0.002, 0.003, 0.005, 0.01, 0.02, 0.03, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.95, 0.99, 0.999, 0.9999]
     plt.yticks(ytickvals)
     plt.gca().set_yticklabels(['{:,.2%}'.format(x) for x in ytickvals])  # formats y ticks as percentage
-    plt.title('Probability plot\nExponential CDF (Weibull Scale)')
-    plt.ylabel('Fraction failing')
     plt.gcf().set_size_inches(9, 7)  # adjust the figsize. This is done post figure creation so that layering is easier
     if show_fitted_distribution is True:
-        plt.plot(xvals, ef, color=color, label=label, **kwargs)
+        ef.CDF(xvals=xvals, label=label, **kwargs) #### remove color
         plt.legend(loc='upper left')
-
+    plt.ylabel('Fraction failing') ####
+    plt.title('Probability plot\nExponential CDF (Weibull Scale)') ####
+    plt.xlabel(xlabel) #### needs to be set after plotting the CDF to override the default 'xvals'
+    return plt.gcf()
 
 def Normal_probability_plot(failures=None, right_censored=None, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
     '''
@@ -399,7 +437,7 @@ def Normal_probability_plot(failures=None, right_censored=None, __fitted_dist_pa
         raise ValueError('Insufficient data to fit a distribution. Minimum number of points is 2')
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)
     plt.ylim([0.0001, 0.9999])
-    plt.gca().set_yscale('function', functions=(__normal_forward, __normal_inverse))
+    plt.gca().set_yscale('function', functions=(axes_transforms.normal_forward, axes_transforms.normal_inverse))
     plt.grid(b=True, which='major', color='k', alpha=0.3, linestyle='-')
     plt.grid(b=True, which='minor', color='k', alpha=0.08, linestyle='-')
     plt.gca().yaxis.set_minor_locator(FixedLocator(np.linspace(0, 1, 51)))
@@ -419,7 +457,7 @@ def Normal_probability_plot(failures=None, right_censored=None, __fitted_dist_pa
     if 'label' in kwargs:
         label = kwargs.pop('label')
     else:
-        label = str('Fitted Normal_2P (μ=' + str(round(mu, 2)) + ', σ=' + str(round(sigma, 2)) + ')')
+        label = str('Fitted Normal_2P (μ=' + str(round_to_decimals(mu, dec)) + ', σ=' + str(round_to_decimals(sigma, dec)) + ')')
     if 'color' in kwargs:
         color = kwargs.pop('color')
         data_color = color
@@ -439,7 +477,7 @@ def Normal_probability_plot(failures=None, right_censored=None, __fitted_dist_pa
     if show_fitted_distribution is True:
         plt.plot(xvals, nf, color=color, label=label, **kwargs)
         plt.legend(loc='upper left')
-
+    return plt.gcf()
 
 def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
     '''
@@ -476,12 +514,12 @@ def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=Fal
         else:
             raise ValueError('right_censored must be a list or an array')
 
-    xmin_log = 10 ** (int(np.floor(np.log10(min(failures)))) - 1)
-    xmax_log = 10 ** (int(np.ceil(np.log10(max(failures)))) + 1)
+    xmin_log = np.floor(np.log10(min(failures))) - 1
+    xmax_log = np.ceil(np.log10(max(failures))) + 1
     if max(failures) < 1:
         xvals = np.linspace(10 ** -3, 2, 1000)
     else:
-        xvals = np.logspace(np.log10(xmin_log) - 2, np.log10(xmax_log) + 2, 1000)
+        xvals = np.logspace(xmin_log - 2, xmax_log + 2, 1000)
 
     if __fitted_dist_params is not None:
         if __fitted_dist_params.gamma > 0:
@@ -500,7 +538,7 @@ def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=Fal
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Lognormal_2P (μ=' + str(round(mu, 2)) + ', σ=' + str(round(sigma, 2)) + ')')
+            label = str('Fitted Lognormal_2P (μ=' + str(round_to_decimals(mu, dec)) + ', σ=' + str(round_to_decimals(sigma, dec)) + ')')
         if 'color' in kwargs:
             color = kwargs.pop('color')
             data_color = color
@@ -523,7 +561,7 @@ def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=Fal
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Lognormal_3P (μ=' + str(round(mu, 2)) + ', σ=' + str(round(sigma, 2)) + ', γ=' + str(round(gamma, 2)) + ')')
+            label = str('Fitted Lognormal_3P (μ=' + str(round_to_decimals(mu, dec)) + ', σ=' + str(round_to_decimals(sigma, dec)) + ', γ=' + str(round_to_decimals(gamma, dec)) + ')')
         if 'color' in kwargs:
             color = kwargs.pop('color')
             data_color = color
@@ -537,7 +575,7 @@ def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=Fal
     # plot the failure points and format the scale and axes
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)
     plt.scatter(x, y, marker='.', linewidth=2, c=data_color)
-    plt.gca().set_yscale('function', functions=(__normal_forward, __normal_inverse))
+    plt.gca().set_yscale('function', functions=(axes_transforms.normal_forward, axes_transforms.normal_inverse))
     plt.xscale('log')
     plt.grid(b=True, which='major', color='k', alpha=0.3, linestyle='-')
     plt.grid(b=True, which='minor', color='k', alpha=0.08, linestyle='-')
@@ -545,8 +583,10 @@ def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=Fal
     xrange = plt.gca().get_xlim()  # this ensures the previously plotted objects are considered when setting the range
     xrange_min = min(min(x), xrange[0])
     xrange_max = max(max(x), xrange[1])
-    pts_min_log = 10 ** (int(np.floor(np.log10(xrange_min))))  # second smallest point is rounded down to nearest power of 10
-    pts_max_log = 10 ** (int(np.ceil(np.log10(xrange_max))))  # largest point is rounded up to nearest power of 10
+    if xrange_min <=0:
+        xrange_min=1e-2
+    pts_min_log = 10 ** (np.floor(np.log10(xrange_min)))  # second smallest point is rounded down to nearest power of 10
+    pts_max_log = 10 ** (np.ceil(np.log10(xrange_max)))  # largest point is rounded up to nearest power of 10
     plt.xlim([pts_min_log, pts_max_log])
     plt.gca().yaxis.set_minor_locator(FixedLocator(np.linspace(0, 1, 51)))
     ytickvals = [0.0001, 0.001, 0.01, 0.03, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.95, 0.97, 0.99, 0.999, 0.9999]
@@ -558,7 +598,7 @@ def Lognormal_probability_plot(failures=None, right_censored=None, fit_gamma=Fal
     if show_fitted_distribution is True:
         plt.plot(xvals, lnf, color=color, label=label, **kwargs)
         plt.legend(loc='upper left')
-
+    return plt.gcf()
 
 def Beta_probability_plot(failures=None, right_censored=None, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
     '''
@@ -599,7 +639,7 @@ def Beta_probability_plot(failures=None, right_censored=None, __fitted_dist_para
     if 'label' in kwargs:
         label = kwargs.pop('label')
     else:
-        label = str('Fitted Beta_2P (α=' + str(round(alpha, 2)) + ', β=' + str(round(beta, 2)) + ')')
+        label = str('Fitted Beta_2P (α=' + str(round_to_decimals(alpha, dec)) + ', β=' + str(round_to_decimals(beta, dec)) + ')')
     if 'color' in kwargs:
         color = kwargs.pop('color')
         data_color = color
@@ -609,8 +649,8 @@ def Beta_probability_plot(failures=None, right_censored=None, __fitted_dist_para
     plt.scatter(x, y, marker='.', linewidth=2, c=data_color)
     if show_fitted_distribution is True:
         bf = Beta_Distribution(alpha=alpha, beta=beta).CDF(show_plot=False, xvals=xvals)
-    f_beta = lambda x: __beta_forward(x, alpha, beta)
-    fi_beta = lambda x: __beta_inverse(x, alpha, beta)
+    f_beta = lambda x: axes_transforms.beta_forward(x, alpha, beta)
+    fi_beta = lambda x: axes_transforms.beta_inverse(x, alpha, beta)
     plt.gca().set_yscale('function', functions=(f_beta, fi_beta))
     plt.plot(xvals, bf, color=color, label=label, **kwargs)
     plt.title('Probability plot\nBeta CDF')
@@ -618,7 +658,7 @@ def Beta_probability_plot(failures=None, right_censored=None, __fitted_dist_para
     plt.ylabel('Fraction failing')
     plt.legend(loc='upper left')
     plt.gcf().set_size_inches(9, 7)  # adjust the figsize. This is done post figure creation so that layering is easier
-
+    return plt.gcf()
 
 def Gamma_probability_plot(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
     '''
@@ -676,7 +716,7 @@ def Gamma_probability_plot(failures=None, right_censored=None, fit_gamma=False,
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Gamma_2P (α=' + str(round(alpha, 2)) + ', β=' + str(round(beta, 2)) + ')')
+            label = str('Fitted Gamma_2P (α=' + str(round_to_decimals(alpha, dec)) + ', β=' + str(round_to_decimals(beta, dec)) + ')')
         if 'color' in kwargs:
             color = kwargs.pop('color')
             data_color = color
@@ -699,7 +739,7 @@ def Gamma_probability_plot(failures=None, right_censored=None, fit_gamma=False,
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Gamma_3P\n(α=' + str(round(alpha, 2)) + ', β=' + str(round(beta, 2)) + ', γ=' + str(round(gamma, 2)) + ')')
+            label = str('Fitted Gamma_3P\n(α=' + str(round_to_decimals(alpha, dec)) + ', β=' + str(round_to_decimals(beta, dec)) + ', γ=' + str(round_to_decimals(gamma, dec)) + ')')
         if 'color' in kwargs:
             color = kwargs.pop('color')
             data_color = color
@@ -713,8 +753,8 @@ def Gamma_probability_plot(failures=None, right_censored=None, fit_gamma=False,
     # plot the failure points and format the scale and axes
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)
     plt.scatter(x, y, marker='.', linewidth=2, c=data_color)
-    f_gamma = lambda x: __gamma_forward(x, beta)
-    fi_gamma = lambda x: __gamma_inverse(x, beta)
+    f_gamma = lambda x: axes_transforms.gamma_forward(x, beta)
+    fi_gamma = lambda x: axes_transforms.gamma_inverse(x, beta)
     plt.gca().set_yscale('function', functions=(f_gamma, fi_gamma))
     plt.grid(b=True, which='major', color='k', alpha=0.3, linestyle='-')
     plt.grid(b=True, which='minor', color='k', alpha=0.08, linestyle='-')
@@ -753,9 +793,9 @@ def Gamma_probability_plot(failures=None, right_censored=None, fit_gamma=False,
     if show_fitted_distribution is True:
         plt.plot(xvals, gf, color=color, label=label, **kwargs)
         plt.legend(loc='upper left')
+    return plt.gcf()
 
-
-def Exponential_probability_plot(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, show_fitted_distribution=True, **kwargs):
+def Exponential_probability_plot(failures=None, right_censored=None, fit_gamma=False, __fitted_dist_params=None, h1=None, h2=None, CI=0.95, show_fitted_distribution=True, **kwargs):
     '''
     Exponential probability plot
 
@@ -767,6 +807,7 @@ def Exponential_probability_plot(failures=None, right_censored=None, fit_gamma=F
     show_fitted_distribution - True/False. If true, the fitted distribution will be plotted on the probability plot. Defaults to True
     h1 and h2 are the heuristic constants for plotting positions of the form (k-h1)/(n+h2). Default is h1=0.3,h2=0.4 which is the median rank method (same as in Minitab).
         For more heuristics, see: https://en.wikipedia.org/wiki/Q%E2%80%93Q_plot#Heuristics
+    CI - the confidence interval for the bounds. Default is 0.95 for 95% CI.
     kwargs are accepted for the fitted line (eg. linestyle, label, color)
 
     Outputs:
@@ -779,6 +820,9 @@ def Exponential_probability_plot(failures=None, right_censored=None, fit_gamma=F
     else:
         xvals = np.logspace(-2, np.ceil(np.log10(max(failures))) + 1, 1000)
 
+    if CI <= 0 or CI >= 1:
+        raise ValueError('CI must be between 0 and 1. Default is 0.95 for 95% Confidence interval.')
+
     if __fitted_dist_params is not None:
         if __fitted_dist_params.gamma > 0:
             fit_gamma = True
@@ -786,52 +830,58 @@ def Exponential_probability_plot(failures=None, right_censored=None, fit_gamma=F
     if fit_gamma is False:
         if __fitted_dist_params is not None:
             Lambda = __fitted_dist_params.Lambda
+            Lambda_SE = __fitted_dist_params.Lambda_SE
         else:
             from reliability.Fitters import Fit_Expon_1P
-            fit = Fit_Expon_1P(failures=failures, right_censored=right_censored, show_probability_plot=False, print_results=False)
+            fit = Fit_Expon_1P(failures=failures, right_censored=right_censored, CI=CI, show_probability_plot=False, print_results=False)
             Lambda = fit.Lambda
-        ef = Exponential_Distribution(Lambda=Lambda).CDF(show_plot=False, xvals=xvals)
+            Lambda_SE = fit.Lambda_SE
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Exponential_1P (λ=' + str(round(Lambda, 8)) + ')')
+            label = str('Fitted Exponential_1P (λ=' + str(round_to_decimals(Lambda, dec)) + ')')
         if 'color' in kwargs:
-            color = kwargs.pop('color')
-            data_color = color
+            data_color = kwargs.get('color')
         else:
-            color = 'red'
             data_color = 'k'
-        plt.xlabel('Time')
+        xlabel = 'Time'
     elif fit_gamma is True:
         if __fitted_dist_params is not None:
             Lambda = __fitted_dist_params.Lambda
+            Lambda_SE = __fitted_dist_params.Lambda_SE
             gamma = __fitted_dist_params.gamma
         else:
             from reliability.Fitters import Fit_Expon_2P
-            fit = Fit_Expon_2P(failures=failures, right_censored=right_censored, show_probability_plot=False, print_results=False)
+            fit = Fit_Expon_2P(failures=failures, right_censored=right_censored, CI=CI, show_probability_plot=False, print_results=False)
             Lambda = fit.Lambda
+            Lambda_SE = fit.Lambda_SE
             gamma = fit.gamma
-        ef = Exponential_Distribution(Lambda=Lambda).CDF(show_plot=False, xvals=xvals)
         if 'label' in kwargs:
             label = kwargs.pop('label')
         else:
-            label = str('Fitted Exponential_2P\n(λ=' + str(round(Lambda, 8)) + ', γ=' + str(round(gamma, 2)) + ')')
+            label = str('Fitted Exponential_2P\n(λ=' + str(round_to_decimals(Lambda, dec)) + ', γ=' + str(round_to_decimals(gamma, dec)) + ')')
         if 'color' in kwargs:
-            color = kwargs.pop('color')
-            data_color = color
+            data_color = kwargs.get('color')
         else:
-            color = 'red'
             data_color = 'k'
-        plt.xlabel('Time - gamma')
+        xlabel = 'Time - gamma'
         failures = failures - gamma
         if right_censored is not None:
             right_censored = right_censored - gamma
+
+        #recalculate the xvals for the plotting range when gamma>0
+        if max(failures) - gamma < 1:
+            xvals = np.logspace(-5, 2, 1000)
+        else:
+            xvals = np.logspace(-4, np.ceil(np.log10(max(failures) - gamma)) + 1, 1000)
+    ef = Exponential_Distribution(Lambda=Lambda, Lambda_SE=Lambda_SE, CI=CI)
+
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)
     plt.scatter(x, y, marker='.', linewidth=2, c=data_color)
     xrange = plt.gca().get_xlim()  # this ensures the previously plotted objects are considered when setting the range
     xrange_max = max(max(x) * 1.2, xrange[1])
     plt.xlim([0, xrange_max])
-    plt.gca().set_yscale('function', functions=(__expon_forward, __expon_inverse))
+    plt.gca().set_yscale('function', functions=(axes_transforms.expon_forward, axes_transforms.expon_inverse))
     plt.grid(b=True, which='major', color='k', alpha=0.3, linestyle='-')
     plt.grid(b=True, which='minor', color='k', alpha=0.08, linestyle='-')
     if max(y) < 0.9:
@@ -860,13 +910,14 @@ def Exponential_probability_plot(failures=None, right_censored=None, fit_gamma=F
         plt.ylim([0.01, 0.9999])
     plt.yticks(ytickvals)
     plt.gca().set_yticklabels(['{:,.2%}'.format(x) for x in ytickvals])  # formats y ticks as percentage
-    plt.title('Probability plot\nExponential CDF')
-    plt.ylabel('Fraction failing')
     plt.gcf().set_size_inches(9, 7)  # adjust the figsize. This is done post figure creation so that layering is easier
     if show_fitted_distribution is True:
-        plt.plot(xvals, ef, color=color, label=label, **kwargs)
+        ef.CDF(xvals=xvals, label=label, **kwargs)
         plt.legend(loc='upper left')
-
+    plt.title('Probability plot\nExponential CDF')
+    plt.ylabel('Fraction failing')
+    plt.xlabel(xlabel)  #### needs to be set after plotting the CDF to override the default 'xvals'
+    return plt.gcf()
 
 def PP_plot_parametric(X_dist=None, Y_dist=None, y_quantile_lines=None, x_quantile_lines=None, show_diagonal_line=False, **kwargs):
     '''
@@ -958,7 +1009,7 @@ def PP_plot_parametric(X_dist=None, Y_dist=None, y_quantile_lines=None, x_quanti
     plt.axis('square')
     plt.xlim([0, 1])
     plt.ylim([0, 1])
-
+    return plt.gcf()
 
 def QQ_plot_parametric(X_dist=None, Y_dist=None, show_fitted_lines=True, show_diagonal_line=False, **kwargs):
     '''
@@ -1150,7 +1201,7 @@ def PP_plot_semiparametric(X_data_failures=None, X_data_right_censored=None, Y_d
     if show_diagonal_line is True:
         plt.plot([0, 1], [0, 1], color='blue', alpha=0.5)
     plt.title('Probability-Probability Plot\nSemi-parametric')
-
+    return plt.gcf()
 
 def QQ_plot_semiparametric(X_data_failures=None, X_data_right_censored=None, Y_dist=None, show_fitted_lines=True, show_diagonal_line=False, method='KM', **kwargs):
     '''
@@ -1261,13 +1312,14 @@ def plot_points(failures=None, right_censored=None, func='CDF', h1=None, h2=None
 
     Plots the failure points as a scatter plot based on the plotting positions.
     This is similar to a probability plot, just without the axes scaling or the fitted distribution.
-    It may be used to overlay the failure points with a fitted distribution on either the CDF, SF, or CHF.
-    It is not possible to plot the points for PDF or HF as this would require integration and lead to a disjointed plot.
+    It may be used to overlay the failure points with a fitted distribution on either the PDF, CDF, SF, HF, or CHF.
+    If you choose to plot the points for PDF or HF the points will not form a smooth curve as this process requires integration of discrete points which leads to a disjointed plot.
+    The PDF and HF points are correct but not as useful as CDF, SF, and CHF.
 
     Inputs:
     failures - an array or list of the failure times. Minimum number of points allowed is 2.
     right_censored -  an array or list of the right censored failure times.
-    func - The distribution function to plot. Choose either 'CDF','SF','CHF'. Default is 'CDF'
+    func - The distribution function to plot. Choose either 'PDF,'CDF','SF','HF','CHF'. Default is 'CDF'
     h1 and h2 are the heuristic constants for plotting positions of the form (k-h1)/(n+h2). Default is h1=0.3,h2=0.4 which is the median rank method (same as the default in Minitab).
         For more heuristics, see: https://en.wikipedia.org/wiki/Q%E2%80%93Q_plot#Heuristics
     kwargs - keyword arguments for the scatter plot. Defaults are set for color='k' and marker='.' These defaults can be changed using kwargs.
@@ -1290,17 +1342,26 @@ def plot_points(failures=None, right_censored=None, func='CDF', h1=None, h2=None
         raise ValueError('failures must be an array or list with at least 2 failure times')
 
     x, y = plotting_positions(failures=failures, right_censored=right_censored, h1=h1, h2=h2)  # get the plotting positions
-    y_adjusted = []
-    if func == 'CDF':
+    y = np.array(y)
+    x = np.array(x)
+
+    if func in ['pdf','PDF']: #the output of this looks messy because the derivative is of discrete points and not a continuous function
+        dy = np.diff(np.hstack([[0],y]))
+        dx = np.diff(np.hstack([[0],x]))
+        y_adjusted = dy / dx  # PDF = dy/dx CDF
+    elif func in ['cdf','CDF']:
         y_adjusted = y
-    elif func == 'SF':
-        for item in y:
-            y_adjusted.append(1 - item)  # SF = 1-CDF
-    elif func == 'CHF':
-        for item in y:
-            y_adjusted.append(-np.log(1 - item))  # CHF = -ln(SF)
+    elif func in ['sf','SF']:
+        y_adjusted = 1 - y  # SF = 1 - CDF
+    elif func in ['hf','HF']: # the output of this looks messy because the derivative is of discrete points and not a continuous function
+        dy = np.diff(np.hstack([[0],-np.log(1-y)]))
+        dx = np.diff(np.hstack([[0], x]))
+        y_adjusted = dy/dx # HF = dy/dx CHF
+    elif func in ['chf','CHF']:
+        y_adjusted = -np.log(1 - y)  # CHF = -ln(SF)
     else:
         raise ValueError('func must be either CDF, SF, or CHF. Default is CDF')
+
     # set plotting defaults for keywords
     if 'color' in kwargs:
         color = kwargs.pop('color')
