@@ -5,6 +5,7 @@ from reliability import Distributions
 from reliability import Fitters
 from reliability import Nonparametric
 from reliability import Other_functions
+from reliability import Reliability_testing
 from reliability import Repairable_systems
 from reliability import Stress_strength
 from reliability import Probability_plotting
@@ -12,3 +13,4 @@ from reliability import Datasets
 from reliability import ALT_fitters
 from reliability import ALT_probability_plotting
 from reliability import PoF
+from reliability import Utils
